import { __decorate } from 'tslib';
import { BehaviorSubject } from 'rxjs';
import { injectable } from 'inversify';

/** Manages and executes commands from all extensions. */
let CommandRegistry = class CommandRegistry {
    constructor() {
        this.entries = new BehaviorSubject([]);
        /** All commands, emitted whenever the set of registered commands changed. */
        this.commands = this.entries;
    }
    registerCommand(entryToRegister) {
        // Enforce uniqueness of command IDs.
        for (const entry of this.entries.value) {
            if (entry.command === entryToRegister.command) {
                throw new Error(`command is already registered: ${JSON.stringify(entry.command)}`);
            }
        }
        this.entries.next([...this.entries.value, entryToRegister]);
        return {
            unsubscribe: () => {
                this.entries.next(this.entries.value.filter(entry => entry !== entryToRegister));
            },
        };
    }
    deregisterCommand(entryToDeregister) {
        this.entries.next(this.entries.value.filter(entry => entry !== entryToDeregister));
        return true;
    }
    executeCommand(parameters) {
        return executeCommand(this.commandsSnapshot, parameters);
    }
    /**
     * The current set of commands. Used by callers that do not need to react to commands being registered or
     * unregistered.
     */
    get commandsSnapshot() {
        return this.entries.value;
    }
};
CommandRegistry = __decorate([
    injectable()
], CommandRegistry);
/**
 * Executes the command (in the commands list) specified in params.
 *
 * Most callers should use CommandRegistry's executeCommand method, which uses the registered commands.
 */
function executeCommand(commands, parameters) {
    const command = commands.find(entry => entry.command === parameters.command);
    if (!command) {
        throw new Error(`command not found: ${JSON.stringify(parameters.command)}`);
    }
    return Promise.resolve(command.run(...(parameters.arguments || [])));
}
// export class CommandService implements ICommandService {
//     private commandService;
//     constructor() {
//         this.commandService = StaticServices.instantiationService.get(ICommandService) as any;
//     }
//     public onWillExecuteCommand = fromEventPattern(this.commandService.onWillExecuteCommand)
//     public _serviceBrand = this.commandService._serviceBrand;
//     public executeCommand(id: string, ...args: any[]) {
//         return this.commandService.executeCommand(id, ...args);
//     }
// }

export { CommandRegistry, executeCommand };
//# sourceMappingURL=command-service.js.map
