import { isErrorLike, combineLatestOrDefault } from '@adminide-stack/extension-api';
import { combineLatest, from, of } from 'rxjs';
import { map, switchMap, catchError, tap, distinctUntilChanged } from 'rxjs/operators/index.js';
import { fromFetch } from 'rxjs/fetch/index.js';
import { isEqual } from 'lodash-es';
import { viewerConfiguredExtensions, isExtensionEnabled, getScriptURLFromExtensionManifest } from '../extensions/extension.js';
import { isDefined } from '../util/types.js';
import { memoizeObservable } from '../util/memorize-observable.js';

/* eslint-disable import/no-extraneous-dependencies */
/* eslint-disable max-classes-per-file */
/* eslint-disable no-use-before-define */
const EHTTPSTATUS = 'HTTPStatusError';
class HTTPStatusError extends Error {
    constructor(response) {
        super(`Request to ${response.url} failed with ${response.status} ${response.statusText}`);
        this.name = EHTTPSTATUS;
        this.status = response.status;
    }
}
/**
 * Checks if a given fetch Response has a HTTP 2xx status code and throws an HTTPStatusError otherwise.
 */
function checkOk(response) {
    if (!response.ok) {
        throw new HTTPStatusError(response);
    }
    return response;
}
const getConfiguredSideloadedExtension = (baseUrl) => fromFetch(`${baseUrl}/package.json`, { selector: (response) => checkOk(response).json() }).pipe(map((response) => ({
    id: response.name,
    manifest: {
        ...response,
        url: `${baseUrl}/${response.main.replace('dist/', '')}`,
    },
    rawManifest: null,
})));
/**
 * Manages the set of extensions that are available and activated.
 *
 * @internal This is an internal implementation detail and is different from the product feature called the
 * "extension registry" (where users can search for and enable extensions).
 */
class ExtensionsService {
    constructor(platformContext, modelService, extensionActivationFilter = extensionsWithMatchedActivationEvent, fetchSideloadedExtension = getConfiguredSideloadedExtension) {
        this.platformContext = platformContext;
        this.modelService = modelService;
        this.extensionActivationFilter = extensionActivationFilter;
        this.fetchSideloadedExtension = fetchSideloadedExtension;
        this.configuredExtensions = viewerConfiguredExtensions({
            settings: this.platformContext.settings,
            requestGraphQL: this.platformContext.requestGraphQL,
        });
        this.memoizedGetScriptURLForExtension = memoizeObservable((url) => asObservable(this.platformContext.getScriptURLForExtension(url)).pipe(catchError((error) => {
            console.error(`Error fetching extension script URL ${url}`, error);
            return [null];
        })), (url) => url);
    }
    /**
     * Returns an observable that emits the set of enabled extensions upon subscription and whenever it changes.
     *
     * Most callers should use {@link ExtensionsService#activeExtensions}.
     */
    get enabledExtensions() {
        return combineLatest([
            from(this.platformContext.settings),
            from(this.configuredExtensions),
            this.sideloadedExtension,
        ]).pipe(map(([settings, configuredExtensions, sideloadedExtension]) => {
            let enabled = configuredExtensions.filter((extension) => isExtensionEnabled(settings, extension.id));
            if (sideloadedExtension) {
                if (!isErrorLike(sideloadedExtension.manifest) && sideloadedExtension.manifest?.publisher) {
                    // Disable extension with the same ID while this extension is sideloaded
                    const constructedID = `${sideloadedExtension.manifest.publisher}/${sideloadedExtension.id}`;
                    enabled = enabled.filter((extension) => extension.id !== constructedID);
                }
                enabled.push(sideloadedExtension);
            }
            return enabled;
        }));
    }
    get sideloadedExtension() {
        return from(this.platformContext.sideloadedExtensionURL).pipe(switchMap((url) => (url ? this.fetchSideloadedExtension(url) : of(null))), catchError((error) => {
            console.error('Error sideloading extension', error);
            return of(null);
        }));
    }
    /**
     * Returns an observable that emits the set of extensions that should be active, based on the previous and
     * current state and each available extenion's activationEvents.
     *
     * An extension is activated when one or more of its activationEvents is true. After an extension has been
     * activated, it remains active for the rest of the sesion (i.e., for as long as the browser tab remains open)
     * as long as it remains enabled. If it is disabled, it is deactivated. (i.e., "activationEvents" are
     * retrospective/sticky.)
     *
     * @todo Consider whether extensions should be deactivated if none of their activationEvents are true (or that
     * plus a certain period of inactivity).
     */
    get activeExtensions() {
        // Extensions that have been activated (including extensions with zero "activationEvents" that evaluate to
        // true currently).
        const activatedExtensionIDs = new Set();
        return combineLatest([from(this.modelService.activeLanguages), this.enabledExtensions]).pipe(tap(([activeLanguages, enabledExtensions]) => {
            const activeExtensions = this.extensionActivationFilter(enabledExtensions, activeLanguages);
            for (const extension of activeExtensions) {
                if (!activatedExtensionIDs.has(extension.id)) {
                    activatedExtensionIDs.add(extension.id);
                    console.log('---ActivatedExntesionsId', activatedExtensionIDs);
                }
            }
        }), map(([, extensions]) => extensions ? extensions.filter((extension) => activatedExtensionIDs.has(extension.id)) : []), distinctUntilChanged((a, b) => isEqual(new Set(a.map((extension) => extension.id)), new Set(b.map((extension) => extension.id)))), switchMap((extensions) => combineLatestOrDefault(extensions.map((extension) => this.memoizedGetScriptURLForExtension(getScriptURLFromExtensionManifest(extension)).pipe(map((scriptURL) => scriptURL === null
            ? null
            : {
                id: extension.id,
                manifest: extension.manifest,
                scriptURL,
            }))))), map((extensions) => extensions.filter(isDefined)), distinctUntilChanged((a, b) => isEqual(new Set(a.map((extension) => extension.id)), new Set(b.map((extension) => extension.id)))));
    }
}
function asObservable(input) {
    return typeof input === 'string' ? of(input) : from(input);
}
function extensionsWithMatchedActivationEvent(enabledExtensions, visibleTextDocumentLanguages) {
    const languageActivationEvents = new Set([...visibleTextDocumentLanguages].map((language) => `onLanguage:${language}`));
    return enabledExtensions.filter((extension) => {
        try {
            if (!extension.manifest) {
                const match = /^cdeops\/lang-(.*)$/.exec(extension.id);
                if (match) {
                    console.warn(`Extension ${extension.id} has been renamed to sourcegraph/${match[1]}. It's safe to remove ${extension.id} from your settings.`);
                }
                else {
                    console.warn(`Extension ${extension.id} was not found. Remove it from settings to suppress this warning.`);
                }
                return false;
            }
            if (isErrorLike(extension.manifest)) {
                console.warn(extension.manifest);
                return false;
            }
            if (!extension.manifest.activationEvents) {
                console.warn(`Extension ${extension.id} has no activation events, so it will never be activated.`);
                return false;
            }
            return extension.manifest.activationEvents.some((event) => event === '*' || languageActivationEvents.has(event));
        }
        catch (error) {
            console.error(error);
        }
        return false;
    });
}

export { ExtensionsService, HTTPStatusError, checkOk };
//# sourceMappingURL=extensions-service.js.map
