import { __decorate } from 'tslib';
import { Subject, BehaviorSubject, throwError } from 'rxjs';
import { ReferenceCounter } from './utils/reference-counter.js';
import { filter, takeWhile, map, startWith } from 'rxjs/operators/index.js';
import { injectable } from 'inversify';

/**
 * Creates a new instance of {@link ModelService}.
 *
 * @sri Updated to class instead of function
 */
let ModelService = class ModelService {
    constructor() {
        /** A map of URIs to TextModels */
        this._models = new Map();
        this._modelUpdates = new Subject();
        this._activeLanguages = new BehaviorSubject(new Set());
        this._languageReferences = new ReferenceCounter();
        this.getModel = (uri) => {
            const model = this.models.get(uri);
            if (!model) {
                throw new Error(`model does not exist with URI ${uri}`);
            }
            return model;
        };
        this.getPartialModel = uri => {
            const { languageId } = this.getModel(uri);
            return {
                languageId,
            };
        };
        this.observeModel = uri => {
            try {
                const model = this.getModel(uri);
                return this.modelUpdates.pipe(filter(updates => updates.some(update => update.uri === uri)), takeWhile(updates => updates.every(update => update.uri !== uri || update.type !== 'deleted')), map(() => this.getModel(uri)), startWith(model));
            }
            catch (error) {
                return throwError(error);
            }
        };
        this.addModel = model => {
            if (this.models.has(model.uri)) {
                throw new Error(`model already exists with URI ${model.uri}`);
            }
            this.models.set(model.uri, model);
            this.modelUpdates.next([{ type: 'added', ...model }]);
            // Update activeLanguages if no other existing model has the same language.
            if (this._languageReferences.increment(model.languageId)) {
                this.activeLanguages.next(new Set(this._languageReferences.keys()));
            }
        };
        this.updateModel = (uri, text) => {
            const model = this.getModel(uri);
            this.models.set(uri, {
                ...model,
                text,
            });
            this.modelUpdates.next([{ type: 'updated', uri, text }]);
        };
        this.hasModel = uri => this.models.has(uri);
        this.removeModel = uri => {
            const model = this.getModel(uri);
            this.models.delete(uri);
            this.modelUpdates.next([{ type: 'deleted', uri }]);
            if (this._languageReferences.decrement(model.languageId)) {
                this.activeLanguages.next(new Set(this._languageReferences.keys()));
            }
        };
    }
    get models() {
        return this._models;
    }
    get modelUpdates() {
        return this._modelUpdates;
    }
    get activeLanguages() {
        return this._activeLanguages;
    }
};
ModelService = __decorate([
    injectable()
], ModelService);

export { ModelService };
//# sourceMappingURL=model-service.js.map
