import { __decorate, __param, __metadata } from 'tslib';
import { Subject, BehaviorSubject, throwError } from 'rxjs';
import { filter, takeWhile, map, startWith } from 'rxjs/operators/index.js';
import { ClientTypes } from '@adminide-stack/extension-api';
import { ReferenceCounter } from './utils/reference-counter.js';
import { injectable, inject } from 'inversify';

const VIEWER_NOT_FOUND_ERROR_NAME = 'ViewerNotFoundError';
class ViewerNotFoundError extends Error {
    constructor(viewerId) {
        super(`Viewer not found: ${viewerId}`);
        this.name = VIEWER_NOT_FOUND_ERROR_NAME;
    }
}
/**
 * Creates a {@link IViewerService} instance.
 *
 * @sri we have converted to class.
 */
let ViewerService = class ViewerService {
    constructor(modelSerivce) {
        this.modelSerivce = modelSerivce;
        this.id = 0;
        this.nextId = () => `viewer#${this.id++}`;
        /** A map of viewer ids to code viewers. */
        this._viewers = new Map();
        this._viewerUpdates = new Subject();
        this._activeViewerUpdates = new BehaviorSubject(undefined);
        /**
         * Returns the Viewer with teh given viewerId.
         * Throws if no viewer exists with the given viewerId.
         */
        this.getViewer = (viewerId) => {
            const viewer = this._viewers.get(viewerId);
            if (!viewer) {
                throw new ViewerNotFoundError(viewerId);
            }
            return viewer;
        };
        this.addViewer = viewerData => {
            const viewerId = this.nextId();
            if (viewerData.type === 'CodeEditor') {
                this.modelReferences.increment(viewerData.resource);
            }
            const viewer = {
                ...viewerData,
                viewerId,
            };
            this._viewers.set(viewerId, viewer);
            this._viewerUpdates.next([{ type: 'added', viewerId, viewerData }]);
            if (viewerData.isActive) {
                this._activeViewerUpdates.next(viewer);
            }
            return viewer;
        };
        this.observeViewer = ({ viewerId }) => {
            try {
                const viewer = this.getViewer(viewerId);
                return this._viewerUpdates.pipe(filter(updates => updates.some(update => update.viewerId === viewerId)), takeWhile(updates => updates.every(update => update.viewerId !== viewerId || update.type !== 'deleted')), map(() => this.getViewer(viewerId)), startWith(viewer));
            }
            catch (error) {
                return throwError(error);
            }
        };
        this.modelReferences = new ReferenceCounter();
    }
    get viewers() {
        return this._viewers;
    }
    get viewerUpdates() {
        return this._viewerUpdates;
    }
    get activeViewerUpdates() {
        return this._activeViewerUpdates;
    }
    setSelections({ viewerId }, selections) {
        const viewer = this.getViewer(viewerId);
        if (viewer.type !== 'CodeEditor') {
            throw new Error(`Editor ID ${viewerId} is type ${String(viewer.type)}, expected CodeExitor`);
        }
        this._viewers.set(viewerId, { ...viewer, selections });
        this._viewerUpdates.next([{ type: 'updated', viewerId, viewerData: { selections } }]);
    }
    removeViewer({ viewerId }) {
        const viewer = this.getViewer(viewerId);
        this._viewers.delete(viewerId);
        this._viewerUpdates.next([{ type: 'deleted', viewerId }]);
        // Check if this was the active viewer
        if (this._activeViewerUpdates.value && this._activeViewerUpdates.value.viewerId === viewerId) {
            this._activeViewerUpdates.next(undefined);
        }
        if (viewer.type === 'CodeEditor' && this.modelReferences.decrement(viewer.resource)) {
            this.modelSerivce.removeModel(viewer.resource);
        }
    }
    removeAllViewers() {
        const updates = [...this._viewers.keys()].map(viewerId => ({ type: 'deleted', viewerId }));
        this._viewers.clear();
        this._viewerUpdates.next(updates);
        // The active viewer will have been removed
        this._activeViewerUpdates.next(undefined);
    }
};
ViewerService = __decorate([
    injectable(),
    __param(0, inject(ClientTypes.IModelService)),
    __metadata("design:paramtypes", [Object])
], ViewerService);

export { ViewerService };
//# sourceMappingURL=viewer-service.js.map
