import { createAggregateError } from './errors.js';
import { parse } from 'jsonc-parser';
import { asError } from '@adminide-stack/extension-api';

/**
 * Parses the JSON input using an error-tolerant "JSONC" parser. If an error occurs, it is returned as a value
 * instead of being thrown. This is useful when input is parased in the background (not in response to any specific
 * user action), because it makes it easier to save the error and only show it to the user when it matters (for
 * some interactive user action).
 */
function parseJSONOrError(input) {
    try {
        return parseJSON(input);
    }
    catch (err) {
        return asError(err);
    }
}
/**
 * Parses the JSON input using an error-tolerant "JSONC" parser.
 */
function parseJSON(input) {
    const errors = [];
    parse(input, errors, { allowTrailingComma: true, disallowComments: false });
    if (errors.length > 0) {
        throw createAggregateError(errors.map(v => ({
            ...v,
            code: printParseErrorCode[v.error],
            message: `parse error (code: ${v.error}, offset: ${v.offset}, length: ${v.length})`,
        })));
    }
}
// until new version of jsonc-parser release
// https://github.com/Microsoft/node-jsonc-parser/issues/14
function printParseErrorCode(code) {
    switch (code) {
        case 1 /* ParseErrorCode.InvalidSymbol */: return 'InvalidSymbol';
        case 2 /* ParseErrorCode.InvalidNumberFormat */: return 'InvalidNumberFormat';
        case 3 /* ParseErrorCode.PropertyNameExpected */: return 'PropertyNameExpected';
        case 4 /* ParseErrorCode.ValueExpected */: return 'ValueExpected';
        case 5 /* ParseErrorCode.ColonExpected */: return 'ColonExpected';
        case 6 /* ParseErrorCode.CommaExpected */: return 'CommaExpected';
        case 7 /* ParseErrorCode.CloseBraceExpected */: return 'CloseBraceExpected';
        case 8 /* ParseErrorCode.CloseBracketExpected */: return 'CloseBracketExpected';
        case 9 /* ParseErrorCode.EndOfFileExpected */: return 'EndOfFileExpected';
        case 10 /* ParseErrorCode.InvalidCommentToken */: return 'InvalidCommentToken';
        case 11 /* ParseErrorCode.UnexpectedEndOfComment */: return 'UnexpectedEndOfComment';
        case 12 /* ParseErrorCode.UnexpectedEndOfString */: return 'UnexpectedEndOfString';
        case 13 /* ParseErrorCode.UnexpectedEndOfNumber */: return 'UnexpectedEndOfNumber';
        case 14 /* ParseErrorCode.InvalidUnicode */: return 'InvalidUnicode';
        case 15 /* ParseErrorCode.InvalidEscapeCharacter */: return 'InvalidEscapeCharacter';
        case 16 /* ParseErrorCode.InvalidCharacter */: return 'InvalidCharacter';
    }
    return '<unknown ParseErrorCode>';
}

export { parseJSONOrError, printParseErrorCode };
//# sourceMappingURL=jsonc.js.map
